@@ -17,7 +17,7 @@ import struct
 import tempfile
 import pkg_resources
 from resilient import ImportDefinition
-from resilient_sdk.util.resilient_objects import DEFAULT_INCIDENT_TYPE_UUID
+from resilient_sdk.util.resilient_objects import DEFAULT_INCIDENT_TYPE_UUID, ResilientObjMap
 from resilient_sdk.util.sdk_exception import SDKException
 from resilient_sdk.util import sdk_helpers
 
@@ -33,20 +33,49 @@ else:
     reload = importlib.reload
 
 # Get the same logger object that is used in app.py
-LOG = logging.getLogger("resilient_sdk_log")
+LOG = logging.getLogger(sdk_helpers.LOGGER_NAME)
 
 # Constants
 BASE_NAME_BUILD = "build"
+BASE_NAME_CUSTOMIZE_PY = "customize.py"
 BASE_NAME_EXTENSION_JSON = "app.json"
 BASE_NAME_EXPORT_RES = "export.res"
-
-PREFIX_EXTENSION_ZIP = "app-"
+BASE_NAME_LOCAL_EXPORT_RES = "export.res"
+BASE_NAME_SETUP_PY = "setup.py"
+BASE_NAME_DIST_DIR = "dist"
+BASE_NAME_DOCKER_FILE = "Dockerfile"
+BASE_NAME_ENTRY_POINT = "entrypoint.sh"
+BASE_NAME_APIKEY_PERMS_FILE = "apikey_permissions.txt"
+BASE_NAME_DOC_DIR = "doc"
+BASE_NAME_README = "README.md"
+BASE_NAME_PAYLOAD_SAMPLES_DIR = "payload_samples"
+BASE_NAME_PAYLOAD_SAMPLES_SCHEMA = "output_json_schema.json"
+BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE = "output_json_example.json"
+BASE_NAME_PAYLOAD_SAMPLES_EX_SUCCESS = "mock_json_expectation_success.json"
+BASE_NAME_PAYLOAD_SAMPLES_EP_SUCCESS = "mock_json_endpoint_success.json"
+BASE_NAME_PAYLOAD_SAMPLES_EX_FAIL = "mock_json_expectation_fail.json"
+BASE_NAME_PAYLOAD_SAMPLES_EP_FAIL = "mock_json_endpoint_fail.json"
 
 PATH_DEFAULT_ICON_EXTENSION_LOGO = pkg_resources.resource_filename("resilient_sdk", "data/ext/icons/app_logo.png")
 PATH_DEFAULT_ICON_COMPANY_LOGO = pkg_resources.resource_filename("resilient_sdk", "data/ext/icons/company_logo.png")
+PATH_DEFAULT_README = pkg_resources.resource_filename("resilient_sdk", "data/codegen/templates/package_template/README.md.jinja2")
+PATH_DEFAULT_SCREENSHOT = pkg_resources.resource_filename("resilient_sdk", "data/codegen/templates/package_template/doc/screenshots/main.png")
+
+PATH_TEMPLATE_PAYLOAD_SAMPLES = "payload_samples/function_name"
+
+PATH_CUSTOMIZE_PY = os.path.join("util", "customize.py")
+PATH_CONFIG_PY = os.path.join("util", "config.py")
+PATH_UTIL_DATA_DIR = os.path.join("util", "data")
+PATH_LOCAL_EXPORT_RES = os.path.join("data", BASE_NAME_LOCAL_EXPORT_RES)
+PATH_SCREENSHOTS = os.path.join(BASE_NAME_DOC_DIR, "screenshots")
+PATH_ICON_EXTENSION_LOGO = os.path.join("icons", "app_logo.png")
+PATH_ICON_COMPANY_LOGO = os.path.join("icons", "company_logo.png")
+
+PREFIX_EXTENSION_ZIP = "app-"
+MIN_SETUP_PY_VERSION = "1.0.0"
 
 SUPPORTED_SETUP_PY_ATTRIBUTE_NAMES = (
-    "author", "name", "version",
+    "author", "name", "display_name", "version",
     "description", "long_description", "url",
     "install_requires", "entry_points"
 )
@@ -218,10 +247,10 @@ def load_customize_py_module(path_customize_py, warn=True):
         temp_customize = u"".join(new_lines)
 
         if sys.version_info.major == 3:
-            temp_file_obj = tempfile.NamedTemporaryFile('w', suffix=".py", encoding="utf8")
+            temp_file_obj = tempfile.NamedTemporaryFile('w', suffix=".py", encoding="utf8", delete=False)
         else:
             temp_customize = temp_customize.encode('utf-8')
-            temp_file_obj = tempfile.NamedTemporaryFile('w+b', suffix=".py")
+            temp_file_obj = tempfile.NamedTemporaryFile('w+b', suffix=".py", delete=False)
 
         try:
             # Write the new temporary customize.py (with resilient-circuits replaced with resilient)
@@ -229,8 +258,9 @@ def load_customize_py_module(path_customize_py, warn=True):
                 temp_file.write(temp_customize)
                 temp_file.flush()
                 module_name = os.path.basename(temp_file.name)[:-3]
-                # Attempt to import the module from the temporary file location.
-                customize_py_module = sdk_helpers.load_py_module(temp_file.name, module_name)
+
+            # Attempt to import the module from the temporary file location.
+            customize_py_module = sdk_helpers.load_py_module(temp_file.name, module_name)
 
         except IOError as ioerr:
            raise IOError("Unexpected IO error '{0}' for file '{1}".format(ioerr, temp_file.name))
@@ -239,6 +269,9 @@ def load_customize_py_module(path_customize_py, warn=True):
             # An an unexpected error trying to load the module temporary customize module.
             raise SDKException(u"Got an error attempting to load temporary customize.py module\n{0}".format(err))
 
+        finally:
+            os.unlink(temp_file.name)
+
     else:
         try:
             customize_py_module = sdk_helpers.load_py_module(path_customize_py, "customize")
@@ -247,9 +280,44 @@ def load_customize_py_module(path_customize_py, warn=True):
 
     return customize_py_module
 
+def remove_default_incident_type_from_import_definition(import_definition):
+    """
+      Take ImportDefinition as input and remove the DEFAULT_INCIDENT_TYPE added by
+       codegen in minify_export.
+    :param import_definition dictionary
+    :return: import_definition dictionary
+    """
+    # Get reference to incident_types if there are any
+    incident_types = import_definition.get("incident_types", [])
+
+    if incident_types:
+
+        incident_type_to_remove = None
+
+        # Loop through and remove this custom one (that is originally added using codegen)
+        for incident_type in incident_types:
+            if incident_type.get("uuid") == DEFAULT_INCIDENT_TYPE_UUID:
+                incident_type_to_remove = incident_type
+                break
+
+        if incident_type_to_remove:
+            incident_types.remove(incident_type_to_remove)
+    return import_definition
 
 def get_import_definition_from_customize_py(path_customize_py_file):
-    """Return the base64 encoded ImportDefinition in a customize.py file as a Dictionary"""
+    """
+       Return the ImportDefinition in a customize.py or /util/data/export.res file as a Dictionary.
+       :param path_customize_py_file: Path to the customize.py file
+       :return import definition dict from /util/data/export.res if the file exists. Otherwise,
+               get it from customize.py
+    """
+
+    # If there is a /util/data/export.res then get the import definition from there.
+    path_src = os.path.dirname(path_customize_py_file)
+    path_local_export_res = os.path.join(path_src, PATH_LOCAL_EXPORT_RES)
+    if os.path.isfile(path_local_export_res):
+        import_definition = get_import_definition_from_local_export_res(path_local_export_res)
+        return import_definition
 
     customize_py = load_customize_py_module(path_customize_py_file)
 
@@ -275,24 +343,21 @@ def get_import_definition_from_customize_py(path_customize_py_file):
     # Get the import defintion as dict
     customize_py_import_definition = customize_py_import_definitions[0]
 
-    # Get reference to incident_types if there are any
-    incident_types = customize_py_import_definition.get("incident_types", [])
-
-    if incident_types:
-
-        incident_type_to_remove = None
-
-        # Loop through and remove this custom one (that is originally added using codegen)
-        for incident_type in incident_types:
-            if incident_type.get("uuid") == DEFAULT_INCIDENT_TYPE_UUID:
-                incident_type_to_remove = incident_type
-                break
-
-        if incident_type_to_remove:
-            incident_types.remove(incident_type_to_remove)
+    # Remove the incident type that was added by codegen that allows the data to import
+    customize_py_import_definition = remove_default_incident_type_from_import_definition(customize_py_import_definition)
 
     return customize_py_import_definition
 
+def get_import_definition_from_local_export_res(path_export_res_file):
+    """Return ImportDefinition from a local (/util/data) export.res file as a Dictionary"""
+
+    # Read /util/data/export.res file
+    import_definition = sdk_helpers.read_json_file(path_export_res_file)
+
+    # Remove the incident type that was added by codegen that allows the data to import
+    import_definition = remove_default_incident_type_from_import_definition(import_definition)
+
+    return import_definition
 
 def get_configs_from_config_py(path_config_py_file):
     """Returns a tuple (config_str, config_list). If no configs found, return ("", []).
@@ -358,8 +423,8 @@ def get_configs_from_config_py(path_config_py_file):
     return (concat_cfg_str, config_list)
 
 def get_apikey_permissions(path):
-    """Returns a list of api keys to allow an integration to run.
-
+    """
+    Returns a list of api keys to allow an integration to run.
     :param path: Location to file with api keys one per line.
     :return apikey_permissions: Return list of api keys.
     """
@@ -452,10 +517,15 @@ def get_icon(icon_name, path_to_icon, width_accepted, height_accepted, default_p
 
 def add_tag(tag_name, list_of_objs):
     """
-    TODO: update this docsting to correct standard
-    Returns list_of_objs with tag_name added to each object
-    """
+    Returns list_of_objs with tag_name added to each object.
+    Replaces any tags that were there originally to address bug INT-3077
 
+    :param tag_name: The name of the tag to add
+    :param list_of_objs: A list of all the objects you want to add the tag too
+    :raise: SDKException: if list_of_objs is corrupt
+    :return: list_of_objs with tag_name added to each object
+    :rtype: list of dicts
+    """
     # Create tag_to_add
     tag_to_add = {
         "tag_handle": tag_name,
@@ -477,23 +547,8 @@ def add_tag(tag_name, list_of_objs):
             LOG.error("Error adding tag.\n'list_of_objs': %s\n'obj': %s", list_of_objs, obj)
             raise SDKException(err_msg.format(tag_name, "obj", "Dictionary", type(obj)))
 
-        # Try get current_tags
-        current_tags = obj.get("tags")
-
-        # If None, create new empty List
-        if current_tags is None:
-            current_tags = []
-
-        # If current_tags is not a list, error
-        if not isinstance(current_tags, list):
-            LOG.error("Error adding tag.\n'current_tags': %s", current_tags)
-            raise SDKException(err_msg.format(tag_name, "current_tags", "List", type(current_tags)))
-
-        # Append our tag_to_add to current_tags
-        current_tags.append(tag_to_add)
-
-        # Set the obj's 'tags' value to current_tags
-        obj["tags"] = current_tags
+        # Set the obj's 'tags' value to tag_to_add
+        obj["tags"] = [tag_to_add]
 
     # Return the updated list_of_objs
     return list_of_objs
@@ -572,25 +627,40 @@ def get_configuration_py_file_path(file_type, setup_py_attributes):
     return path_py_file
 
 def create_extension(path_setup_py_file, path_apikey_permissions_file,
-                     output_dir, path_built_distribution=None, path_extension_logo=None, path_company_logo=None,
-                     custom_display_name=None, repository_name=None, keep_build_dir=False):
+                     output_dir, path_built_distribution=None, path_extension_logo=None, path_company_logo=None, path_payload_samples=None,
+                     custom_display_name=None, repository_name=None, image_hash=None, keep_build_dir=False):
     """
-    TODO: update this docstring to new standard format
     Function that creates The App.zip file from the given setup.py, customize and config files
-    and copies it to the output_dir. Returns the path to the App.zip
-    - path_setup_py_file [String]: abs path to the setup.py file
-    - path_apikey_permissions_file [String]: abs path to the apikey_permissions.txt file
-    - output_dir [String]: abs path to the directory the App.zip should be produced
-    - path_built_distribution [String]: abs path to a tar.gz Built Distribution
+    and copies it to the output_dir. Returns the path to the app.zip
+
+    :param path_setup_py_file: abs path to the setup.py file
+    :type path_setup_py_file: str
+    :param path_apikey_permissions_file: abs path to the apikey_permissions.txt file
+    :type path_apikey_permissions_file: str
+    :param output_dir: abs path to the directory the App.zip should be produced
+    :type output_dir: str
+    :param path_built_distribution: abs path to a tar.gz Built Distribution
         - if provided uses that .tar.gz
         - else looks for it in the output_dir. E.g: output_dir/package_name.tar.gz
-    - path_extension_logo [String]: abs path to the app_logo.png. Has to be 200x72 and a .png file
+    :type path_built_distribution: str
+    :param path_extension_logo: abs path to the app_logo.png. Has to be 200x72 and a .png file
         - if not provided uses default icon
-    - path_company_logo [String]: abs path to the company_logo.png. Has to be 100x100 and a .png file
+    :type path_extension_logo: str
+    :param path_company_logo: abs path to the company_logo.png. Has to be 100x100 and a .png file
         - if not provided uses default icon
-    - custom_display_name [String]: will give the App that display name. Default: name from setup.py file
-    - repository_name [String]: will over-ride the container repository name for the App. Default: 'ibmresilient'
-    - keep_build_dir [Boolean]: if True, build/ will not be remove. Default: False
+    :type path_company_logo: str
+    :param path_payload_samples: abs path to directory containing the files with a JSON schema and example output of the functions
+    :type path_payload_samples: str
+    :param custom_display_name: will give the App that display name. Default: name from setup.py file
+    :type custom_display_name: str
+    :param repository_name: will over-ride the container repository name for the App. Default: 'ibmresilient'
+    :type repository_name: str
+    :param image_hash: if defined will append the hash to the image_name in the app.json file e.g. <repository_name>/<package_name>@sha256:<image_hash>. Default: <repository_name>/<package_name>:<version>
+    :type image_hash: str
+    :param keep_build_dir: if True, build/ will not be removed. Default: False
+    :type keep_build_dir: bool
+    :return: Path to new app.zip
+    :rtype: str
     """
 
     LOG.info("Creating App")
@@ -627,22 +697,22 @@ def create_extension(path_setup_py_file, path_apikey_permissions_file,
 
     # Get the customize file location.
     path_customize_py_file = get_configuration_py_file_path("customize", setup_py_attributes)
-  
+
     # Get the config file location.
     path_config_py_file = get_configuration_py_file_path("config", setup_py_attributes)
 
     # Get ImportDefinition from the discovered customize file.
     if path_customize_py_file:
-        customize_py_import_definition = get_import_definition_from_customize_py(path_customize_py_file)
+        import_definition = get_import_definition_from_customize_py(path_customize_py_file)
     else:
         # No 'customize.py' file found generate import definition with just mimimum server version.
-        customize_py_import_definition = {
+        import_definition = {
             'server_version':
                 IMPORT_MIN_SERVER_VERSION
         }
 
     # Add the tag to the import defintion
-    customize_py_import_definition = add_tag_to_import_definition(tag_name, SUPPORTED_RES_OBJ_NAMES, customize_py_import_definition)
+    import_definition = add_tag_to_import_definition(tag_name, SUPPORTED_RES_OBJ_NAMES, import_definition)
 
     # Parse the app.configs from the discovered config file
     if path_config_py_file:
@@ -702,6 +772,27 @@ def create_extension(path_setup_py_file, path_apikey_permissions_file,
             height_accepted=100,
             default_path_to_icon=PATH_DEFAULT_ICON_COMPANY_LOGO)
 
+        # Get the display name
+        # Use --display-name if passed
+        # If not use 'display_name' attribute in setup.py
+        # If not set use the 'name' attribute in setup.py
+        display_name = custom_display_name or setup_py_attributes.get("display_name") or setup_py_attributes.get("name")
+
+        # Image string is all lowercase on quay.io
+        image_name = "{0}/{1}:{2}".format(repository_name, setup_py_attributes.get("name"), setup_py_attributes.get("version"))
+
+        if image_hash:
+
+            if not sdk_helpers.is_valid_hash(image_hash):
+                raise SDKException(u"image_hash '{0}' is not a valid SHA256 hash\nIt must be a valid hexadecimal and 64 characters long".format(image_hash))
+
+            # If image_hash is defined append to image name e.g. <repository_name>/<package_name>@sha256:<image_hash>
+            image_name = "{0}/{1}@sha256:{2}".format(repository_name, setup_py_attributes.get("name"), image_hash)
+
+        image_name = image_name.lower()
+
+        LOG.debug("image_name generated: %s", image_name)
+
         # Generate the contents for the extension.json file
         the_extension_json_file_contents = {
             "author": {
@@ -716,7 +807,7 @@ def create_extension(path_setup_py_file, path_apikey_permissions_file,
                 "content": setup_py_attributes.get("description"),
                 "format": "text"
             },
-            "display_name": custom_display_name if custom_display_name is not None else setup_py_attributes.get("name"),
+            "display_name": display_name,
             "icon": {
                 "data": extension_logo,
                 "media_type": "image/png"
@@ -726,10 +817,10 @@ def create_extension(path_setup_py_file, path_apikey_permissions_file,
                 "format": "html"
             },
             "minimum_resilient_version": {
-                "major": customize_py_import_definition.get("server_version").get("major", None),
-                "minor": customize_py_import_definition.get("server_version").get("minor", None),
-                "build_number": customize_py_import_definition.get("server_version").get("build_number", None),
-                "version": customize_py_import_definition.get("server_version").get("version", None)
+                "major": import_definition.get("server_version").get("major", None),
+                "minor": import_definition.get("server_version").get("minor", None),
+                "build_number": import_definition.get("server_version").get("build_number", None),
+                "version": import_definition.get("server_version").get("version", None)
             },
             "name": setup_py_attributes.get("name"),
             "tag": {
@@ -744,8 +835,7 @@ def create_extension(path_setup_py_file, path_apikey_permissions_file,
                 "executables": [
                     {
                         "name": setup_py_attributes.get("name"),
-                        "image": "{0}/{1}:{2}".format(repository_name, setup_py_attributes.get("name"),
-                                                      setup_py_attributes.get("version")),
+                        "image": image_name,
                         "config_string": app_configs[0],
                         "permission_handles": apikey_permissions,
                         "uuid": uuid
@@ -757,13 +847,49 @@ def create_extension(path_setup_py_file, path_apikey_permissions_file,
         # Write the executable.json file
         sdk_helpers.write_file(path_extension_json, json.dumps(the_extension_json_file_contents, sort_keys=True))
 
-        # Write the customize ImportDefinition to the export.res file
-        sdk_helpers.write_file(path_export_res, json.dumps(customize_py_import_definition, sort_keys=True))
+        # See if RES_SDK_DEV environment var is set
+        if sdk_helpers.is_env_var_set(sdk_helpers.ENV_VAR_DEV):
+
+            if not path_payload_samples:
+                LOG.warning("WARNING: No path for 'payload_samples' provided. Skipping adding them to the export.res file")
+
+            else:
+                for fn in import_definition.get("functions"):
+
+                    # Get paths to payload_samples
+                    fn_name = fn.get(ResilientObjMap.FUNCTIONS)
+                    path_payload_samples_fn = os.path.join(path_payload_samples, fn_name)
+                    path_payload_samples_schema = os.path.join(path_payload_samples_fn, BASE_NAME_PAYLOAD_SAMPLES_SCHEMA)
+                    path_payload_samples_example = os.path.join(path_payload_samples_fn, BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE)
+
+                    try:
+                        # Validate payload_files, add custom error message if we can't
+                        sdk_helpers.validate_file_paths(os.R_OK, path_payload_samples_schema, path_payload_samples_example)
+                    except SDKException as err:
+                        err.message += ("\nWARNING: could not access JSON file to add payload_samples. Continuing to create package.\n"
+                                        "Add '--no-samples' flag to avoid looking for them and avoid this warning message.\n")
+                        LOG.warning(err.message)
+                        continue
+
+                    # Read in schema payload and add to function import definition
+                    payload_samples_schema_contents_dict = sdk_helpers.read_json_file(path_payload_samples_schema)
+                    LOG.debug("Adding JSON output schema to '%s' from file: %s", fn_name, path_payload_samples_schema)
+                    json_schema_key = os.path.splitext(BASE_NAME_PAYLOAD_SAMPLES_SCHEMA)[0]
+                    fn[json_schema_key] = json.dumps(payload_samples_schema_contents_dict)
+
+                    # Read in example payload and add to function import definition
+                    payload_samples_example_contents_dict = sdk_helpers.read_json_file(path_payload_samples_example)
+                    LOG.debug("Adding JSON output example to '%s' from file: %s", fn_name, path_payload_samples_example)
+                    json_example_key = os.path.splitext(BASE_NAME_PAYLOAD_SAMPLES_EXAMPLE)[0]
+                    fn[json_example_key] = json.dumps(payload_samples_example_contents_dict)
+
+        # Write the customize ImportDefinition to the app*.zip export.res file
+        sdk_helpers.write_file(path_export_res, json.dumps(import_definition, sort_keys=True))
 
         # Copy the built distribution to the build dir, enforce rename to .tar.gz
         shutil.copy(path_built_distribution, os.path.join(path_build, "{0}.tar.gz".format(extension_name)))
 
-        # create The Extension Zip by zipping the build directory
+        # Create the app.zip (Extension Zip) by zipping the build directory
         extension_zip_base_path = os.path.join(output_dir, "{0}{1}".format(PREFIX_EXTENSION_ZIP, extension_name))
         extension_zip_name = shutil.make_archive(base_name=extension_zip_base_path, format="zip", root_dir=path_build)
         path_the_extension_zip = os.path.join(extension_zip_base_path, extension_zip_name)
